#include <glibmm/refptr.h>

#include <cassert>
#include <iostream>
#include "CommandSample.h"

using Glib::RefPtr;

class AddCommand : public Command
{
public:
  AddCommand(Gtk::Entry* aInputField,
             Gtk::ListViewText* aListView)
    : mInputField(aInputField)
    , mListView(aListView)
  { }

  void Execute() override
  {
    std::string text = mInputField->get_text();

    if (! text.empty()) {
      std::cout << "Adding " << text << "\n";

      mListView->append(text);

      mInputField->set_text("");
    }
  }

private:
  Gtk::Entry* mInputField;
  Gtk::ListViewText* mListView;
};

class DeleteCommand : public Command
{
public:
  DeleteCommand(Gtk::Entry* aInputField,
                Gtk::ListViewText* aListView)
    : mInputField(aInputField)
    , mListView(aListView)
  { }

  void Execute() override
  {
    std::vector<int> selections = mListView->get_selected();
    assert(selections.size() <= 1);

    if (selections.size() > 0) {
      int selectedRow = selections[0];
      std::cout << "Deleting " << mListView->get_text(selectedRow) << std::endl;

      RefPtr<Gtk::ListStore> model =
        RefPtr<Gtk::ListStore>::cast_dynamic(mListView->get_model());

      if (model) {
        auto iterator = model->children()[selectedRow];
        model->erase(iterator);
      }
    }
  }

private:
  Gtk::Entry* mInputField;
  Gtk::ListViewText* mListView;
};

AppWindow::AppWindow()
  : mAddButton(Gtk::manage(new Gtk::Button()))
  , mDeleteButton(Gtk::manage(new Gtk::Button()))
  , mListView(/*columns_count = */ 1)
{
  set_title("Mediator Sample");
  set_default_size(440, 480);
  move(100, 100);

  add(mFixedContainer);
  mFixedContainer.add(*mAddButton);
  mFixedContainer.add(*mDeleteButton);
  mFixedContainer.add(mInputField);
  mFixedContainer.add(mListView);

  mAddButton->set_label("Add");
  mAddButton->set_size_request(80, 20);
  mDeleteButton->set_label("Delete");
  mDeleteButton->set_size_request(80, 20);

  mListView.set_column_title(0, "Items");

  mInputField.set_size_request(200, 20);
  mListView.set_size_request(400, 400);

  mFixedContainer.move(mInputField, 20, 20);
  mFixedContainer.move(*mAddButton, 240, 20);
  mFixedContainer.move(*mDeleteButton, 340, 20);
  mFixedContainer.move(mListView, 20, 60);

  SetupEventHandlers();

  mFixedContainer.show_all();
}

void
AppWindow::SetupEventHandlers()
{
  // FIXME The command instances are leaked.
  // Fix this this, disconnect() the signal handlers and delete them when they
  // are no longer used (e.g. when the Window is closed).

  // For GTKMM signals, see https://developer.gnome.org/gtkmm-tutorial/stable/sec-connecting-signal-handlers.html.en

  AddCommand* addCommand = new AddCommand(&mInputField, &mListView);
  mAddButton->signal_clicked().connect(
    sigc::mem_fun(addCommand, // The instance to call on_command() with
                  &AddCommand::on_command //
                  ));

  DeleteCommand* deleteCommand = new DeleteCommand(&mInputField, &mListView);
  mDeleteButton->signal_clicked().connect(
    sigc::mem_fun(deleteCommand,
                  &DeleteCommand::on_command)
  );

}

int main(int argc, char *argv[])
{
  auto app =
    Gtk::Application::create(argc, argv,
      "patterns.mvc");

  AppWindow window;

  return app->run(window);
}
